
// __________________________________________________________
//
//                         w2k_pnt.c
//         Windows NT/2000 Processes'n'Threads V1.00
//                06-18-2001 Sven B. Schreiber
//                       sbs@orgon.com
// __________________________________________________________

#include "w2k_pnt.h"

// =================================================================
// DISCLAIMER
// =================================================================

/*

This software is provided "as is" and any express or implied
warranties, including, but not limited to, the implied warranties of
merchantability and fitness for a particular purpose are disclaimed.
In no event shall the author Sven B. Schreiber be liable for any
direct, indirect, incidental, special, exemplary, or consequential
damages (including, but not limited to, procurement of substitute
goods or services; loss of use, data, or profits; or business
interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software,
even if advised of the possibility of such damage.

*/

// =================================================================
// REVISION HISTORY
// =================================================================

/*

06-18-2001 V1.00 Original version (SBS).

*/

// =================================================================
// GLOBAL STRINGS
// =================================================================

WORD awArguments [] = L"[/p] [/t] { [<filter>] }";

// =================================================================
// HELPER FUNCTIONS
// =================================================================

DWORD SystemVersion (VOID)
    {
    OSVERSIONINFO ovi;
    DWORD         dVersion = 0;

    ovi.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);

    if (GetVersionEx (&ovi) &&
        (ovi.dwPlatformId == VER_PLATFORM_WIN32_NT))
        {
        dVersion = ovi.dwMajorVersion;
        }
    return dVersion;
    }

// -----------------------------------------------------------------

PSYSTEM_PROCESS_INFORMATION ProcessInformation (PDWORD    pdData,
                                                PNTSTATUS pns)
    {
    DWORD                       dSize;
    DWORD                       dData = 0;
    NTSTATUS                    ns    = STATUS_INVALID_PARAMETER;
    PSYSTEM_PROCESS_INFORMATION pspi  = NULL;

    for (dSize = 0x1000; (pspi == NULL) && dSize; dSize <<= 1)
        {
        if ((pspi = LocalAlloc (LMEM_FIXED, dSize)) == NULL)
            {
            ns = STATUS_NO_MEMORY;
            break;
            }
        ns = NtQuerySystemInformation (SystemProcessInformation,
                                       pspi, dSize, &dData);
        if (ns != STATUS_SUCCESS)
            {
            LocalFree (pspi);

            pspi  = NULL;
            dData = 0;

            if (ns != STATUS_INFO_LENGTH_MISMATCH) break;
            }
        }
    if (pdData != NULL) *pdData = dData;
    if (pns    != NULL) *pns    = ns;
    return pspi;
    }

// -----------------------------------------------------------------

BOOL WINAPI PatternMatcher (PWORD pwFilter,
                            PWORD pwData,
                            BOOL  fIgnoreCase)
    {
    DWORD i = 0;
    DWORD j = 0;

    if (pwFilter == NULL) return TRUE;
    if (pwData   == NULL) return FALSE;

    while (pwFilter [i] && pwData [j])
        {
        if (pwFilter [i] != '?')
            {
            if (pwFilter [i] == '*')
                {
                i++;

                if ((pwFilter [i] != '*') &&
                    (pwFilter [i] != '?'))
                    {
                    if (pwFilter [i])
                        {
                        while (pwData [j]
                               &&
                               (!PatternMatcher (pwFilter + i,
                                                 pwData   + j,
                                                 fIgnoreCase)))
                            {
                            j++;
                            }
                        }
                    return pwData [j] != 0;
                    }
                }
            if (fIgnoreCase
                ? LCASE (pwFilter [i]) != LCASE (pwData [j])
                :        pwFilter [i]  !=        pwData [j] )
                {
                return FALSE;
                }
            }
        i++;
        j++;
        }
    if (pwFilter [i] == '*') i++;
    return !(pwFilter [i] || pwData [j]);
    }

// =================================================================
// DISPLAY FUNCTIONS
// =================================================================

VOID DisplayProcesses (PSYSTEM_PROCESS pspFirst,
                       PWORD           pwFilter)
    {
    DWORD           dProcesses = 0;
    DWORD           dThreads   = 0;
    PSYSTEM_PROCESS psp        = pspFirst;
    WORD            awTime []  = L"mm-dd HH:MM";
    PFILETIME       pft;
    FILETIME        ft;
    SYSTEMTIME      st;
    PWORD           pwName;

    printf (L"\r\n"
            L"  PID  PPID  KBytes PFaults Handles Threads  "
            L"Date & Time  Name\r\n"
            L"----------------------------------------"
            L"---------------------------------------\r\n");

    while (psp != NULL)
        {
        pwName = (psp->dUniqueProcessId
                  ? psp->usName.Buffer
                  : L"Idle");

        if (PatternMatcher (pwFilter, pwName, TRUE))
            {
            lstrcpy (awTime, L"  ?     ?  ");
            pft = &psp->ftCreateTime;

            if ((pft->dwLowDateTime || pft->dwHighDateTime) &&
                FileTimeToLocalFileTime (pft, &ft)          &&
                FileTimeToSystemTime    (&ft, &st))
                {
                sprintf (awTime, L"%02hu-%02hu %02hu:%02hu",
                         st.wMonth, st.wDay,
                         st.wHour,  st.wMinute);
                }
            printf (L"%5lu %5lu %7lu %7lu %7lu %7lu  "
                    L"%-11s  %s\r\n",
                    psp->dUniqueProcessId,
                    psp->dInheritedFromUniqueProcessId,
                    psp->VmCounters.WorkingSetSize / 1024,
                    psp->VmCounters.PageFaultCount,
                    psp->dHandleCount,
                    psp->dThreadCount,
                    awTime,
                    pwName);

            dProcesses += 1;
            dThreads   += psp->dThreadCount;
            }
        psp = (psp->dNext
               ? (PVOID) ((PBYTE) psp + psp->dNext)
               : NULL);
        }
    printf (L"%s----------------------------------------"
            L"---------------------------------------\r\n"
            L"%5lu process%s, %lu thread%s\r\n",
            (dProcesses ? L"" : L"+++ No matching names +++\r\n"),
            dProcesses, (dProcesses == 1 ? L"" : L"es"),
            dThreads,   (dThreads   == 1 ? L"" : L"s" ));
    return;
    }

// -----------------------------------------------------------------

VOID DisplayThreads (PSYSTEM_THREAD pstFirst,
                     DWORD          dThreads)
    {
    PSYSTEM_THREAD pst           = pstFirst;
    WORD           awTime []     = L"mm-dd HH:MM";
    DWORDLONG      dlTotalKernel = 0;
    DWORDLONG      dlTotalUser   = 0;
    DWORDLONG      dlKernel, dlUser;
    PFILETIME      pft;
    FILETIME       ft;
    SYSTEMTIME     st;
    DWORD          i;

    printf (L"\r\n"
            L"  TID Prio Base   Switches  Address  State  "
            L"Date & Time  Kernel(sec)  User(sec)\r\n"
            L"----------------------------------------"
            L"---------------------------------------\r\n");

    for (i = 0; i < dThreads; i++)
        {
        lstrcpy (awTime, L"  ?     ?  ");
        pft = &pst->ftCreateTime;

        if ((pft->dwLowDateTime || pft->dwHighDateTime) &&
            FileTimeToLocalFileTime (pft, &ft)          &&
            FileTimeToSystemTime    (&ft, &st))
            {
            sprintf (awTime, L"%02hu-%02hu %02hu:%02hu",
                     st.wMonth, st.wDay,
                     st.wHour,  st.wMinute);
            }
        dlKernel   = pst->ftKernelTime.dwHighDateTime;
        dlKernel <<= 32;
        dlKernel  += pst->ftKernelTime.dwLowDateTime;
        dlKernel  /= 10000;

        dlUser     = pst->ftUserTime.dwHighDateTime;
        dlUser   <<= 32;
        dlUser    += pst->ftUserTime.dwLowDateTime;
        dlUser    /= 10000;

        dlTotalKernel += dlKernel;
        dlTotalUser   += dlUser;

        printf (L"%5lu %4lu %4lu %10lu  %08lX %2lu/%02lu  "
                L"%-11s %8lu.%03lu %6lu.%03lu\r\n",
                pst->Cid.UniqueThread,
                pst->dPriority,
                pst->dBasePriority,
                pst->dContextSwitches,
                pst->pStartAddress,
                pst->dThreadState,
                pst->WaitReason,
                awTime,
                (DWORD) dlKernel / 1000,
                (DWORD) dlKernel % 1000,
                (DWORD) dlUser   / 1000,
                (DWORD) dlUser   % 1000);
        pst++;
        }
    printf (L"----------------------------------------"
            L"---------------------------------------\r\n"
            L"%5lu thread%s"
            L"                                           "
            L"%8lu.%03lu %6lu.%03lu\r\n",
            dThreads, (dThreads == 1 ? L" " : L"s"),
            (DWORD) dlTotalKernel / 1000,
            (DWORD) dlTotalKernel % 1000,
            (DWORD) dlTotalUser   / 1000,
            (DWORD) dlTotalUser   % 1000);
    return;
    }

// -----------------------------------------------------------------

VOID DisplayData (PWORD pwFilter,
                  BOOL  fProcesses,
                  BOOL  fThreads,
                  BOOL  fNT4)
    {
    PSYSTEM_PROCESS_INFORMATION pspi;
    PSYSTEM_PROCESS             psp;
    PSYSTEM_THREAD              pst;
    PWORD                       pwName, pwFilter1;
    NTSTATUS                    ns;
    DWORD                       dData, dProcesses, dThreads, i;

    printf (L"\r\nWindows %s mode\r\n",
            (fNT4 ? L"NT 4.0" : L"2000"));

    if ((pspi = ProcessInformation (&dData, &ns)) != NULL)
        {
        psp       = &pspi->Process;
        pwFilter1 = (pwFilter != NULL ? pwFilter : L"*");

        if (fProcesses)
            {
            printf (L"\r\n"
                    L"PROCESS INFO (Filter: '%s')\r\n"
                    L"=========================",
                    pwFilter1);

            for (i = lstrlen (pwFilter1); i; i--) printf (L"=");
            printf (L"\r\n");

            DisplayProcesses (psp, pwFilter1);
            }
        if (fThreads)
            {
            printf (L"\r\n"
                    L"THREAD INFO (Filter: '%s')\r\n"
                    L"========================",
                    pwFilter1);

            for (i = lstrlen (pwFilter1); i; i--) printf (L"=");
            printf (L"\r\n");

            dProcesses = 0;
            dThreads   = 0;

            while (psp != NULL)
                {
                pwName = (psp->dUniqueProcessId
                          ? psp->usName.Buffer
                          : L"Idle");

                if (PatternMatcher (pwFilter1, pwName, TRUE))
                    {
                    printf (L"\r\nPROCESS #%lu (%s)\r\n",
                            psp->dUniqueProcessId, pwName);

                    pst = (fNT4
                           ? ((PSYSTEM_PROCESS_NT4) psp)->aThreads
                           : ((PSYSTEM_PROCESS_NT5) psp)->aThreads);

                    DisplayThreads (pst, psp->dThreadCount);

                    dProcesses += 1;
                    dThreads   += psp->dThreadCount;
                    }
                psp = (psp->dNext
                       ? (PVOID) ((PBYTE) psp + psp->dNext)
                       : NULL);
                }
            printf (L"\r\nTOTAL: %lu process%s, %lu thread%s\r\n",
                    dProcesses, (dProcesses == 1 ? L"" : L"es"),
                    dThreads,   (dThreads   == 1 ? L"" : L"s" ));
            }
        LocalFree (pspi);
        }
    else
        {
        printf (L"\r\nNT error code = 0x%08lX", ns);
        }
    return;
    }

// =================================================================
// MAIN PROGRAM
// =================================================================

DWORD Main (DWORD argc, PWORD *argv, PWORD *argp)
    {
    DWORD dVersion = SystemVersion ();
    DWORD i, j;
    BOOL  fProcesses, fThreads;
    PWORD pwFilter;

    printf (atAbout);

    if (argc < 2)
        {
        printf (atUsage, awArguments);
        }
    else
        {
        fProcesses = FALSE;
        fThreads   = FALSE;

        for (i = 1; i < argc; i++)
            {
            pwFilter = NULL;

            for (j = 0; (pwFilter == NULL) && argv [i] [j]; j++)
                {
                if (argv [i] [j] == '/')
                    {
                    switch (LCASE (argv [i] [j+1]))
                        {
                        case 'p': fProcesses = TRUE; j += 1; break;
                        case 't': fThreads   = TRUE; j += 1; break;
                        default:  pwFilter   = argv [i] + j; break;
                        }
                    }
                else
                    {
                    pwFilter = argv [i] + j;
                    }
                }
            if ((pwFilter != NULL) || (i+1 == argc))
                {
                if (dVersion >= 4)
                    {
                    DisplayData (pwFilter, fProcesses, fThreads,
                                 dVersion == 4);
                    }
                else
                    {
                    printf (L"\r\nOS version not supported\r\n");
                    break;
                    }
                }
            }
        }
    return 0;
    }

// =================================================================
// END OF PROGRAM
// =================================================================
