
// __________________________________________________________
//
//                         w2k_sys.h
//           Windows 2000 System Information V1.00
//                06-18-2001 Sven B. Schreiber
//                       sbs@orgon.com
// __________________________________________________________

#ifndef _W2K_SYS_H_
#define _W2K_SYS_H_

// =================================================================
// DISCLAIMER
// =================================================================

/*

This software is provided "as is" and any express or implied
warranties, including, but not limited to, the implied warranties of
merchantibility and fitness for a particular purpose are disclaimed.
In no event shall the author Sven B. Schreiber be liable for any
direct, indirect, incidental, special, exemplary, or consequential
damages (including, but not limited to, procurement of substitute
goods or services; loss of use, data, or profits; or business
interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software,
even if advised of the possibility of such damage.

*/

// =================================================================
// REVISION HISTORY
// =================================================================

/*

06-18-2001 V1.00 Original version (SBS).

*/

// =================================================================
// THREAD STATES
// =================================================================

#define THREAD_STATE_INITIALIZED             0
#define THREAD_STATE_READY                   1
#define THREAD_STATE_RUNNING                 2
#define THREAD_STATE_STANDBY                 3
#define THREAD_STATE_TERMINATED              4
#define THREAD_STATE_WAIT                    5
#define THREAD_STATE_TRANSITION              6
#define THREAD_STATE_UNKNOWN                 7

// =================================================================
// STATUS CODES
// =================================================================

#define STATUS_SUCCESS              ((NTSTATUS) 0x00000000)
#define STATUS_INFO_LENGTH_MISMATCH ((NTSTATUS) 0xC0000004)
#define STATUS_INVALID_PARAMETER    ((NTSTATUS) 0xC000000D)

// =================================================================
// SIMPLE TYPES
// =================================================================

typedef LONG KPRIORITY, *PKPRIORITY, **PPKPRIORITY;
typedef LONG NTSTATUS,  *PNTSTATUS,  **PPNTSTATUS;

// =================================================================
// ENUMERATIONS
// =================================================================

typedef enum _KWAIT_REASON
        {
/*000*/ Executive,
/*001*/ FreePage,
/*002*/ PageIn,
/*003*/ PoolAllocation,
/*004*/ DelayExecution,
/*005*/ Suspended,
/*006*/ UserRequest,
/*007*/ WrExecutive,
/*008*/ WrFreePage,
/*009*/ WrPageIn,
/*00A*/ WrPoolAllocation,
/*00B*/ WrDelayExecution,
/*00C*/ WrSuspended,
/*00D*/ WrUserRequest,
/*00E*/ WrEventPair,
/*00F*/ WrQueue,
/*010*/ WrLpcReceive,
/*011*/ WrLpcReply,
/*012*/ WrVirtualMemory,
/*013*/ WrPageOut,
/*014*/ WrRendezvous,
/*015*/ Spare2,
/*016*/ Spare3,
/*017*/ Spare4,
/*018*/ Spare5,
/*019*/ Spare6,
/*01A*/ WrKernel,
/*01B*/ MaximumWaitReason
        }
        KWAIT_REASON,
     * PKWAIT_REASON,
    **PPKWAIT_REASON;

// =================================================================
// STRUCTURES
// =================================================================

typedef struct _CLIENT_ID
        {
/*000*/ HANDLE UniqueProcess;
/*004*/ HANDLE UniqueThread;
/*008*/ }
        CLIENT_ID,
     * PCLIENT_ID,
    **PPCLIENT_ID;

#define CLIENT_ID_ sizeof (CLIENT_ID)

// -----------------------------------------------------------------

typedef struct _UNICODE_STRING
        {
/*000*/ WORD  Length;
/*002*/ WORD  MaximumLength;
/*004*/ PWORD Buffer;
/*008*/ }
        UNICODE_STRING,
     * PUNICODE_STRING,
    **PPUNICODE_STRING;

#define UNICODE_STRING_ sizeof (UNICODE_STRING)

// -----------------------------------------------------------------

typedef struct _VM_COUNTERS
        {
/*000*/ DWORD PeakVirtualSize;
/*004*/ DWORD VirtualSize;
/*008*/ DWORD PageFaultCount;
/*00C*/ DWORD PeakWorkingSetSize;
/*010*/ DWORD WorkingSetSize;
/*014*/ DWORD QuotaPeakPagedPoolUsage;
/*018*/ DWORD QuotaPagedPoolUsage;
/*01C*/ DWORD QuotaPeakNonPagedPoolUsage;
/*020*/ DWORD QuotaNonPagedPoolUsage;
/*024*/ DWORD PagefileUsage;
/*028*/ DWORD PeakPagefileUsage;
/*02C*/ }
        VM_COUNTERS,
     * PVM_COUNTERS,
    **PPVM_COUNTERS;

#define VM_COUNTERS_ sizeof (VM_COUNTERS)

// =================================================================
// SYSTEM INFO CLASSES
// =================================================================

typedef enum _SYSTEMINFOCLASS
    {
    SystemBasicInformation,             // 0x002C
    SystemProcessorInformation,         // 0x000C
    SystemPerformanceInformation,       // 0x0138
    SystemTimeInformation,              // 0x0020
    SystemPathInformation,              // not implemented
    SystemProcessInformation,           // 0x00F8+ per process
    SystemCallInformation,              // 0x0018 + (n * 0x0004)
    SystemConfigurationInformation,     // 0x0018
    SystemProcessorCounters,            // 0x0030 per cpu
    SystemGlobalFlag,                   // 0x0004
    SystemInfo10,                       // not implemented
    SystemModuleInformation,            // 0x0004 + (n * 0x011C)
    SystemLockInformation,              // 0x0004 + (n * 0x0024)
    SystemInfo13,                       // not implemented
    SystemPagedPoolInformation,         // checked build only
    SystemNonPagedPoolInformation,      // checked build only
    SystemHandleInformation,            // 0x0004  + (n * 0x0010)
    SystemObjectInformation,            // 0x0038+ + (n * 0x0030+)
    SystemPagefileInformation,          // 0x0018+ per page file
    SystemInstemulInformation,          // 0x0088
    SystemInfo20,                       // invalid info class
    SystemCacheInformation,             // 0x0024
    SystemPoolTagInformation,           // 0x0004 + (n * 0x001C)
    SystemProcessorStatistics,          // 0x0000, or 0x0018 per cpu
    SystemDpcInformation,               // 0x0014
    SystemMemoryUsageInformation1,      // checked build only
    SystemLoadImage,                    // 0x0018, set mode only
    SystemUnloadImage,                  // 0x0004, set mode only
    SystemTimeAdjustmentInformation,    // 0x000C, 0x0008 writeable
    SystemMemoryUsageInformation2,      // checked build only
    SystemInfo30,                       // checked build only
    SystemInfo31,                       // checked build only
    SystemCrashDumpInformation,         // 0x0004
    SystemExceptionInformation,         // 0x0010
    SystemCrashDumpStateInformation,    // 0x0008
    SystemDebuggerInformation,          // 0x0002
    SystemThreadSwitchInformation,      // 0x0030
    SystemRegistryQuotaInformation,     // 0x000C
    SystemLoadDriver,                   // 0x0008, set mode only
    SystemPrioritySeparationInformation,// 0x0004, set mode only
    SystemInfo40,                       // not implemented
    SystemInfo41,                       // not implemented
    SystemInfo42,                       // invalid info class
    SystemInfo43,                       // invalid info class
    SystemTimeZoneInformation,          // 0x00AC
    SystemLookasideInformation,         // n * 0x0020

// info classes specific to Windows 2000
// WTS = Windows Terminal Server

    SystemSetTimeSlipEvent,             // set mode only
    SystemCreateSession,                // WTS, set mode only
    SystemDeleteSession,                // WTS, set mode only
    SystemInfo49,                       // invalid info class
    SystemRangeStartInformation,        // 0x0004
    SystemVerifierInformation,          // 0x0068
    SystemAddVerifier,                  // set mode only
    SystemSessionProcessesInformation,  // WTS
    }
    SYSTEMINFOCLASS,
 * PSYSTEMINFOCLASS,
**PPSYSTEMINFOCLASS;

// =================================================================
// SYSTEM INFO CLASS #05 - SystemProcessInformation
// =================================================================

typedef struct _SYSTEM_THREAD
        {
/*000*/ FILETIME     ftKernelTime;      // 100 nsec units
/*008*/ FILETIME     ftUserTime;        // 100 nsec units
/*010*/ FILETIME     ftCreateTime;      // relative to 01-01-1601
/*018*/ DWORD        dWaitTime;
/*01C*/ PVOID        pStartAddress;
/*020*/ CLIENT_ID    Cid;               // process/thread ids
/*028*/ DWORD        dPriority;
/*02C*/ DWORD        dBasePriority;
/*030*/ DWORD        dContextSwitches;
/*034*/ DWORD        dThreadState;      // 2=running, 5=waiting
/*038*/ KWAIT_REASON WaitReason;
/*03C*/ DWORD        dReserved01;
/*040*/ }
        SYSTEM_THREAD,
     * PSYSTEM_THREAD,
    **PPSYSTEM_THREAD;

#define SYSTEM_THREAD_ sizeof (SYSTEM_THREAD)

// -----------------------------------------------------------------

typedef struct _SYSTEM_PROCESS          // common members
        {
/*000*/ DWORD          dNext;           // relative offset
/*004*/ DWORD          dThreadCount;
/*008*/ DWORD          dReserved01;
/*00C*/ DWORD          dReserved02;
/*010*/ DWORD          dReserved03;
/*014*/ DWORD          dReserved04;
/*018*/ DWORD          dReserved05;
/*01C*/ DWORD          dReserved06;
/*020*/ FILETIME       ftCreateTime;    // relative to 01-01-1601
/*028*/ FILETIME       ftUserTime;      // 100 nsec units
/*030*/ FILETIME       ftKernelTime;    // 100 nsec units
/*038*/ UNICODE_STRING usName;
/*040*/ KPRIORITY      BasePriority;
/*044*/ DWORD          dUniqueProcessId;
/*048*/ DWORD          dInheritedFromUniqueProcessId;
/*04C*/ DWORD          dHandleCount;
/*050*/ DWORD          dReserved07;
/*054*/ DWORD          dReserved08;
/*058*/ VM_COUNTERS    VmCounters;      // see ntddk.h
/*084*/ DWORD          dCommitCharge;   // bytes
/*088*/ }
        SYSTEM_PROCESS,
     * PSYSTEM_PROCESS,
    **PPSYSTEM_PROCESS;

#define SYSTEM_PROCESS_ sizeof (SYSTEM_PROCESS)

// -----------------------------------------------------------------

typedef struct _SYSTEM_PROCESS_NT4      // Windows NT 4.0
        {
/*000*/ SYSTEM_PROCESS Process;         // common members
/*088*/ SYSTEM_THREAD  aThreads [];     // thread array
/*088*/ }
        SYSTEM_PROCESS_NT4,
     * PSYSTEM_PROCESS_NT4,
    **PPSYSTEM_PROCESS_NT4;

#define SYSTEM_PROCESS_NT4_ sizeof (SYSTEM_PROCESS_NT4)

// -----------------------------------------------------------------

typedef struct _SYSTEM_PROCESS_NT5      // Windows 2000
        {
/*000*/ SYSTEM_PROCESS Process;         // common members
/*088*/ IO_COUNTERS    IoCounters;      // see ntddk.h
/*0B8*/ SYSTEM_THREAD  aThreads [];     // thread array
/*0B8*/ }
        SYSTEM_PROCESS_NT5,
     * PSYSTEM_PROCESS_NT5,
    **PPSYSTEM_PROCESS_NT5;

#define SYSTEM_PROCESS_NT5_ sizeof (SYSTEM_PROCESS_NT5)

// -----------------------------------------------------------------

typedef union _SYSTEM_PROCESS_INFORMATION
        {
/*000*/ SYSTEM_PROCESS     Process;
/*000*/ SYSTEM_PROCESS_NT4 Process_NT4;
/*000*/ SYSTEM_PROCESS_NT5 Process_NT5;
/*0B8*/ }
        SYSTEM_PROCESS_INFORMATION,
     * PSYSTEM_PROCESS_INFORMATION,
    **PPSYSTEM_PROCESS_INFORMATION;

// =================================================================
// ntdll.dll API PROTOTYPES
// =================================================================

NTSTATUS NTAPI
NtQuerySystemInformation (SYSTEMINFOCLASS SystemInformationClass,
                          PVOID           SystemInformation,
                          DWORD           SystemInformationLength,
                          PDWORD          ReturnLength OPTIONAL);

NTSTATUS NTAPI
NtSetSystemInformation   (SYSTEMINFOCLASS SystemInformationClass,
                          PVOID           SystemInformation,
                          DWORD           SystemInformationLength);

// =================================================================
// LINKER CONTROL
// =================================================================

#pragma comment (linker, "/defaultlib:ntdll.lib")

// =================================================================
// END OF MAIN #ifndef CLAUSE
// =================================================================

#endif // #ifndef _W2K_SYS_H_

// =================================================================
// END OF FILE
// =================================================================
